@@ -265,6 +265,9 @@ enum litest_device_type {
 	LITEST_UCLOGIC_TABLET,
 	LITEST_KEYBOARD_BLADE_STEALTH,
 	LITEST_KEYBOARD_BLADE_STEALTH_VIDEOSWITCH,
+	LITEST_WACOM_BAMBOO_2FG_PAD,
+	LITEST_WACOM_BAMBOO_2FG_PEN,
+	LITEST_WACOM_BAMBOO_2FG_FINGER,
 };
 
 enum litest_device_feature {
